package impl

import (
	"context"
	"fmt"
	"strings"

	"github.com/sfreiberg/gotwilio"
	"gitlab.mashroom.com/mashroom/backend/common/api/grpcerr"
	"gitlab.mashroom.com/mashroom/backend/common/generic/env"
	"gitlab.mashroom.com/mashroom/backend/twilio/model"
	"gitlab.mashroom.com/mashroom/backend/twilio/voice"
)

const (
	MobileCode   = "44"
	MobilePrefix = "+" + MobileCode
)

var (
	Client                    *gotwilio.Twilio
	SiteURL                   = env.ResolveString("SITE_URL", "https://dev.env.mashroom.com")
	TwilioAccountSid          = env.ResolveString("TWILIO_ACCOUNT_SID", "AC14345a5251bc1e898df95ba92ef89ee0")
	TwilioAuthToken           = env.ResolveString("TWILIO_AUTH_TOKEN", "5e51e4682d6490159a6bfa8c91468bd7")
	TwilioMessagingServiceSid = env.ResolveString("TWILIO_MESSAGING_SERVICE_SID", "MG448a070149c53143068e7af2594b6240")
)

func init() {
	Client = gotwilio.NewTwilioClient(TwilioAccountSid, TwilioAuthToken)
}

func VoiceFileHandlerImpl(file string) ([]byte, error) {
	buf, err := voice.Asset(file)
	if err != nil {
		return nil, grpcerr.New(grpcerr.NotFound, "binary file not found")
	}

	return buf, nil
}

func VoiceHandlerImpl(ctx context.Context, data []byte) ([]byte, error) {
	bodyStr := string(data)
	req := strings.Split(bodyStr, "&")

	callSid := model.VoiceGetReqParam(req, "CallSid")
	fmt.Printf("callSid = %s\n", callSid)
	if callSid == "" {
		return nil, grpcerr.New(grpcerr.InvalidData, "forbidden for callSid empty")
	}

	ivrModel, err := model.GetIVRModelByCallSid(ctx, callSid)
	if err != nil {
		fmt.Printf("error GetIVRModelByCallSid = %v\n", err)
		return nil, err
	}
	if ivrModel == nil {
		ivrModel = &model.IVRModel{
			CallSid:    callSid,
			State:      model.IvrStateWelcome,
			Properties: make([]string, 0),
		}

		err = model.InsertIVRModel(ctx, ivrModel)
		if err != nil {
			fmt.Printf("error InsertIVRModel = %v\n", err)
			return nil, err
		}
	}

	var x []byte

	switch ivrModel.State {
	case model.IvrStateWelcome:
		ivrModel.State = model.IvrStateWelcomeResponse
		err = model.UpdateIVRModel(ctx, ivrModel)
		if err != nil {
			return nil, err
		}
		x, err = model.VoiceGetMessage(model.IvrStateWelcome)
	case model.IvrStateWelcomeResponse:
		digit := model.VoiceGetReqParam(req, "Digits")
		x, err = VoiceHandleInitialOptions(ctx, ivrModel, digit)
	case model.IvrStateEnterPropertyRefResponse:
		digit := model.VoiceGetReqParam(req, "Digits")
		var prop string
		prop, err = model.VoiceGetProperty(ctx, digit)
		if err != nil {
			if ivrModel.State == model.IvrStateEnterPropertyRefRepeat {
				if model.VoiceInOfficeHours() {
					x, err = model.VoiceConnectToSupport()
				} else {
					x, err = model.VoiceGetMessage(model.IvrStatePropertyRefIncorrect)
				}
			} else {
				ivrModel.State = model.IvrStateEnterPropertyRefRepeat
				_ = model.UpdateIVRModel(ctx, ivrModel)
				x, err = model.VoiceGetMessage(model.IvrStateEnterPropertyRefRepeat)
			}
		} else {
			ivrModel.Properties = append(ivrModel.Properties, prop)
			ivrModel.State = model.IvrStateMobileCorrect
			err = model.UpdateIVRModel(ctx, ivrModel)
			if err != nil {
				return nil, err
			}
			x, err = model.VoiceGetMessage(model.IvrStateMobileCorrect)
		}
	case model.IvrStateMobileCorrect:
		digit := model.VoiceGetReqParam(req, "Digits")
		switch digit {
		case "1": // correct
			mobileNumber := model.VoiceGetReqParam(req, "Caller")
			mobileNumber, err = VoiceParseMobileNumber(mobileNumber)
			if err != nil {
				ivrModel.State = model.IvrStateMobile
				err = model.UpdateIVRModel(ctx, ivrModel)
				if err != nil {
					return nil, err
				}
				x, err = model.VoiceGetMessage(model.IvrStateMobile)
			} else {
				err = VoiceSendSMS(mobileNumber, ivrModel.Properties)
				if err != nil {
					ivrModel.State = model.IvrStateMobile
					err = model.UpdateIVRModel(ctx, ivrModel)
					if err != nil {
						return nil, err
					}
					x, err = model.VoiceGetMessage(model.IvrStateMobile)
				} else {
					ivrModel.State = model.IvrStateBookAgain
					err = model.UpdateIVRModel(ctx, ivrModel)
					if err != nil {
						return nil, err
					}
					x, err = model.VoiceGetMessage(model.IvrStateBookAgain)
				}
			}
		case "2": // not correct
			ivrModel.State = model.IvrStateMobile
			err = model.UpdateIVRModel(ctx, ivrModel)
			if err != nil {
				return nil, err
			}
			x, err = model.VoiceGetMessage(model.IvrStateMobile)
		default:
			x, err = model.VoiceGetMessage(model.IvrStateMobileCorrect)
		}
	case model.IvrStateMobile:
		mobileNumber := model.VoiceGetReqParam(req, "Digits")
		mobileNumber, err = VoiceParseMobileNumber(mobileNumber)
		if err != nil {
			x, err = model.VoiceGetMessage(model.IvrStateMobile)
		} else {
			err = VoiceSendSMS(mobileNumber, ivrModel.Properties)
			if err != nil {
				x, err = model.VoiceGetMessage(model.IvrStateMobile)
			} else {
				x, _ = model.VoiceGetMessage(model.IvrStateBookAgain)
				ivrModel.State = model.IvrStateBookAgain
				err = model.UpdateIVRModel(ctx, ivrModel)
				if err != nil {
					return nil, err
				}
			}
		}
	case model.IvrStateBookAgain:
		digit := model.VoiceGetReqParam(req, "Digits")
		if digit == "1" { // yes
			ivrModel.Properties = make([]string, 0)
			ivrModel.State = model.IvrStateEnterPropertyRefResponse
			err = model.UpdateIVRModel(ctx, ivrModel)
			if err != nil {
				return nil, err
			}
			x, err = model.VoiceGetMessage(model.IvrStateEnterPropertyRef)
		} else {
			if model.VoiceInOfficeHours() {
				ivrModel.State = model.IvrStateAnotherQuery
				err = model.UpdateIVRModel(ctx, ivrModel)
				if err != nil {
					return nil, err
				}
				x, err = model.VoiceGetMessage(model.IvrStateAnotherQuery)
			} else {
				x, err = model.VoiceGetMessage(model.IvrStateOutro)
				_ = model.DeleteIVRModelByCallSid(ctx, ivrModel.CallSid)
			}
		}
	case model.IvrStateAnotherQuery:
		digit := model.VoiceGetReqParam(req, "Digits")
		if digit == "1" { // yes
			x, err = model.VoiceConnectToSupport()
		} else {
			x, err = model.VoiceGetMessage(model.IvrStateOutro)
			_ = model.DeleteIVRModelByCallSid(ctx, ivrModel.CallSid)
		}
	}

	if err != nil {
		return nil, err
	}

	return x, nil
}

func VoiceHandleInitialOptions(ctx context.Context, ivrModel *model.IVRModel, choice string) ([]byte, error) {
	switch choice {
	case "1":
		// Book a viewing
		ivrModel.State = model.IvrStateEnterPropertyRefResponse
		err := model.UpdateIVRModel(ctx, ivrModel)
		if err != nil {
			return nil, err
		}
		return model.VoiceGetMessage(model.IvrStateEnterPropertyRef)
	case "2":
		// More info on property
		ivrModel.State = model.IvrStateAnotherQuery
		err := model.UpdateIVRModel(ctx, ivrModel)
		if err != nil {
			return nil, err
		}
		return model.VoiceGetMessage(model.IvrStateMorePropertyInfo)
	case "3":
		// List a property
		if model.VoiceInOfficeHours() {
			return model.VoiceConnectToSupport()
		}
		return model.VoiceGetMessage(model.IvrStateHowToList)
	case "4":
		// Other
		if model.VoiceInOfficeHours() {
			return model.VoiceConnectToSupport()
		}
		return model.VoiceGetMessage(model.IvrStateOutroOutOfHours)
	case "5":
		// Repeat
		ivrModel.State = model.IvrStateWelcome
		err := model.UpdateIVRModel(ctx, ivrModel)
		if err != nil {
			return nil, err
		}
		return model.VoiceGetMessage(model.IvrStateWelcome)
	}
	return nil, grpcerr.New(grpcerr.InvalidData, "unrecognised option")
}

func VoiceParseMobileNumber(mobileNumber string) (string, error) {
	if len(mobileNumber) == 0 {
		return "", grpcerr.New(grpcerr.InvalidData, "empty mobile number")
	}

	mobileNumber = strings.Replace(mobileNumber, "%2B", "+", 1)

	if string(mobileNumber[0]) == "+" {
		return mobileNumber, nil
	}
	if len(mobileNumber) == 11 && string(mobileNumber[0]) == "7" {
		mobileNumber = "+" + mobileNumber
		return mobileNumber, nil
	}

	if string(mobileNumber[0]) == "0" {
		if len(mobileNumber) >= 10 {
			mobileNumber = MobilePrefix + mobileNumber[1:]
		} else {
			mobileNumber = MobilePrefix + mobileNumber
		}
	} else if strings.HasPrefix(mobileNumber, MobileCode) {
		if len(mobileNumber) >= 11 {
			mobileNumber = "+" + strings.Replace(mobileNumber, MobileCode, "", 1)
		} else {
			mobileNumber = MobilePrefix + mobileNumber
		}
	}

	return mobileNumber, nil
}

func VoiceSendSMS(mobileNumber string, prop []string) error {
	body := "Book your viewing by clicking on this link: "
	for _, p := range prop {
		link := fmt.Sprintf("https://%s/property/%s", SiteURL, p)
		body = body + link + ","
	}
	body = body[:len(body)-1]

	statusCallback := "" // not used for now
	applicationSid := "" // not used for now
	_, exception, err := Client.SendSMSWithCopilot(TwilioMessagingServiceSid, mobileNumber, body, statusCallback, applicationSid)
	if exception != nil {
		return grpcerr.Newf(grpcerr.ProviderError, exception.Error())
	}
	return err
}
