package model

import (
	"context"
	"encoding/xml"
	"fmt"
	"strings"
	"time"

	"gitlab.mashroom.com/mashroom/backend/common/api/grpcerr"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"

	"gitlab.mashroom.com/mashroom/backend/common/generic/env"
	"gitlab.mashroom.com/mashroom/backend/common/generic/grpc"
	"gitlab.mashroom.com/mashroom/backend/common/generic/log"
	"gitlab.mashroom.com/mashroom/backend/common/generic/wrap"
	property "gitlab.mashroom.com/mashroom/backend/property/api"
)

type IvrStateEnum int32

const (
	IvrStateWelcome                  IvrStateEnum = 0
	IvrStateWelcomeResponse          IvrStateEnum = 1
	IvrStateEnterPropertyRef         IvrStateEnum = 2
	IvrStateEnterPropertyRefResponse IvrStateEnum = 3
	IvrStateEnterPropertyRefRepeat   IvrStateEnum = 4
	IvrStatePropertyRefIncorrect     IvrStateEnum = 5
	IvrStatePropertyRef              IvrStateEnum = 6
	IvrStateMobileCorrect            IvrStateEnum = 7
	IvrStateMobile                   IvrStateEnum = 8
	IvrStateSmsResponse              IvrStateEnum = 9
	IvrStateBookAgain                IvrStateEnum = 10
	IvrStateMorePropertyInfo         IvrStateEnum = 11
	IvrStateTalkToSupport            IvrStateEnum = 12
	IvrStateHowToList                IvrStateEnum = 13
	IvrStateAnotherQuery             IvrStateEnum = 14
	IvrStateOutro                    IvrStateEnum = 15
	IvrStateOutroOutOfHours          IvrStateEnum = 16
)

var (
	IvrState = map[IvrStateEnum]string{
		IvrStateWelcome:                "welcome-message",
		IvrStateEnterPropertyRef:       "enter-propref",
		IvrStateEnterPropertyRefRepeat: "ID-incorrect",
		IvrStatePropertyRefIncorrect:   "ID-incorrect-twice",
		IvrStateMorePropertyInfo:       "info-for-property",
		IvrStateMobile:                 "book-mobile",
		IvrStateMobileCorrect:          "book-mobile-corect",
		IvrStateBookAgain:              "book-again",
		IvrStateAnotherQuery:           "other-queries-working-hours",
		IvrStateHowToList:              "How-to-list-outside-hours",
		IvrStateOutro:                  "thanks-for-calling",
		IvrStateOutroOutOfHours:        "outro-out-of-hours",
	}
)

var (
	PropertyServiceURL = env.ResolveString("PROPERTY_SERVICE_URL", "mashroom-property:10000")
	PropertyService    = property.NewPropertyServiceClient(grpc.Dial(PropertyServiceURL, "Property"))

	SiteURL  = env.ResolveString("SITE_URL", "dev.env.mashroom.com")
	MediaURL = "https://" + SiteURL + "/callback/twilio/file/"
)

type IVRModel struct {
	ID         primitive.ObjectID `bson:"_id"`
	CallSid    string             `bson:"call_sid"`
	State      IvrStateEnum       `bson:"state"`
	Properties []string           `bson:"property"`
}

type Gather struct {
	Timeout   string `xml:"timeout,attr"`
	NumDigits string `xml:"numDigits,attr"`
	Play      Play   `xml:""`
}

type Play struct {
	Loop string `xml:"loop,attr"`
	Text string `xml:",innerxml"`
}

type Dial struct {
	Text string `xml:",innerxml"`
}

type TML struct {
	XMLName xml.Name `xml:"Response"`
	Gather  Gather   `xml:""`
}

type TmlLeave struct {
	XMLName xml.Name `xml:"Response"`
	Play    Play     `xml:""`
}

type TmlForward struct {
	XMLName xml.Name `xml:"Response"`
	Dial    Dial     `xml:""`
}

type TmlChain struct {
	XMLName xml.Name `xml:"Response"`
	Play    []Play   `xml:""`
	Gather  Gather   `xml:""`
}

func InsertIVRModel(ctx context.Context, ivr *IVRModel) error {
	fmt.Printf("insert model with callSid = %s\n", ivr.CallSid)
	ivr.ID = primitive.NewObjectID()
	_, err := Twilios.InsertOne(ctx, ivr)
	return err
}

func GetIVRModelByCallSid(ctx context.Context, callSid string) (*IVRModel, error) {
	fmt.Printf("get model by callSid = %s\n", callSid)

	count, _ := Twilios.CountDocuments(ctx, bson.M{"call_sid": callSid})
	if count <= 0 {
		return nil, nil
	}

	var result = &IVRModel{}
	err := Twilios.FindOne(ctx, bson.M{"call_sid": callSid}).Decode(result)
	if err != nil {
		return nil, err
	}
	return result, nil
}

func UpdateIVRModel(ctx context.Context, mn *IVRModel) error {
	log.Println(mn)
	_, err := Twilios.ReplaceOne(ctx, bson.M{"_id": mn.ID}, mn)
	if err != nil {
		fmt.Printf("error UpdateIVRModel = %v\n", err)
	}
	return err
}

func DeleteIVRModel(ctx context.Context, mn *IVRModel) error {
	log.Println(mn)
	_, err := Twilios.DeleteOne(ctx, bson.M{"_id": mn.ID})
	return err
}

func DeleteIVRModelByCallSid(ctx context.Context, callSid string) error {
	_, err := Twilios.DeleteOne(ctx, bson.M{"call_sid": callSid})
	if err != nil {
		fmt.Printf("error DeleteIVRModelByCallSid = %v\n", err)
	}
	return err
}

func VoiceInOfficeHours() bool {
	var officeStartHour int
	var officeStartMinute int
	var officeEndHour int
	var officeEndMinute int

	timestamp := time.Now()
	switch timestamp.Weekday() {
	case time.Sunday:
		return false
	case time.Saturday:
		officeStartHour = env.ResolveInt("OFFICE_WEEKEND_START_HOUR", 9)
		officeStartMinute = env.ResolveInt("OFFICE_WEEKEND_START_MINUTE", 0)
		officeEndHour = env.ResolveInt("OFFICE_WEEKEND_END_HOUR", 16)
		officeEndMinute = env.ResolveInt("OFFICE_WEEKEND_END_MINUTE", 0)
	default:
		officeStartHour = env.ResolveInt("OFFICE_WEEKDAY_START_HOUR", 9)
		officeStartMinute = env.ResolveInt("OFFICE_WEEKDAY_START_MINUTE", 0)
		officeEndHour = env.ResolveInt("OFFICE_WEEKDAY_END_HOUR", 17)
		officeEndMinute = env.ResolveInt("OFFICE_WEEKDAY_END_MINUTE", 0)
	}
	officeStart := time.Date(timestamp.Year(), timestamp.Month(), timestamp.Day(), officeStartHour, officeStartMinute, 0, 0, time.Local)
	officeEnd := time.Date(timestamp.Year(), timestamp.Month(), timestamp.Day(), officeEndHour, officeEndMinute, 0, 0, time.Local)
	if timestamp.Before(officeStart) || timestamp.After(officeEnd) {
		return false
	}
	return true
}

func (state *IvrStateEnum) VoiceGetMedia() (string, error) {
	if media, ok := IvrState[*state]; ok {
		return media + ".mp3", nil
	}
	return "", grpcerr.New(grpcerr.InvalidData, "media not found")
}

func VoiceGetMessage(state IvrStateEnum) ([]byte, error) {
	media, err := state.VoiceGetMedia()
	if err != nil {
		return nil, err
	}

	switch state {
	case IvrStateMorePropertyInfo:
		inHours := VoiceInOfficeHours()
		var state2 IvrStateEnum
		if inHours {
			state2 = IvrStateAnotherQuery
		} else {
			state2 = IvrStateOutroOutOfHours
		}
		media2, err := state2.VoiceGetMedia()
		if err != nil {
			return nil, err
		}
		var tml TmlChain
		if inHours {
			tml = TmlChain{Play: make([]Play, 1)}
			tml.Play = append(tml.Play, Play{Loop: "1", Text: MediaURL + media})
			tml.Gather = Gather{Play: Play{Loop: "1", Text: MediaURL + media2}}
		} else {
			tml = TmlChain{Play: make([]Play, 2)}
			tml.Play = append(tml.Play, Play{Loop: "1", Text: MediaURL + media})
			tml.Play = append(tml.Play, Play{Loop: "1", Text: MediaURL + media2})
		}
		x, err := xml.Marshal(tml)
		log.WithFields(log.Fields{"tml": tml, "xml": string(x), "err": err}).Info("Property Info")
		if err != nil {
			return nil, err
		}
		return x, nil
	case IvrStateEnterPropertyRef, IvrStateEnterPropertyRefRepeat:
		tml := TML{Gather: Gather{Play: Play{Loop: "1", Text: MediaURL + media}, Timeout: "15", NumDigits: "6"}}
		x, err := xml.Marshal(tml)
		if err != nil {
			return nil, err
		}
		return x, nil
	case IvrStatePropertyRefIncorrect:
		tml := TmlLeave{Play: Play{Loop: "1", Text: MediaURL + media}}
		x, err := xml.Marshal(tml)
		if err != nil {
			return nil, err
		}
		return x, nil
	case IvrStateMobile:
		tml := TML{Gather: Gather{Play: Play{Loop: "1", Text: MediaURL + media}, Timeout: "15", NumDigits: "11"}}
		x, err := xml.Marshal(tml)
		if err != nil {
			return nil, err
		}
		return x, nil
	case IvrStateHowToList:
		state2 := IvrStateOutroOutOfHours
		media2, err := state2.VoiceGetMedia()
		if err != nil {
			return nil, err
		}
		tml := TmlChain{Play: make([]Play, 2)}
		tml.Play = append(tml.Play, Play{Loop: "1", Text: MediaURL + media})
		tml.Play = append(tml.Play, Play{Loop: "1", Text: MediaURL + media2})
		x, err := xml.Marshal(tml)
		if err != nil {
			return nil, err
		}
		return x, nil
	case IvrStateOutro:
		tml := TmlLeave{Play: Play{Loop: "1", Text: MediaURL + media}}
		x, err := xml.Marshal(tml)
		if err != nil {
			return nil, err
		}
		return x, nil
	case IvrStateOutroOutOfHours:
		tml := TmlLeave{Play: Play{Loop: "1", Text: MediaURL + media}}
		x, err := xml.Marshal(tml)
		if err != nil {
			return nil, err
		}
		return x, nil
	default:
		tml := TML{Gather: Gather{Play: Play{Loop: "1", Text: MediaURL + media}, Timeout: "15", NumDigits: "1"}}
		x, err := xml.Marshal(tml)
		if err != nil {
			return nil, err
		}
		return x, nil
	}
}

func VoiceConnectToSupport() ([]byte, error) {
	tml := TmlForward{Dial: Dial{Text: env.ResolveString("SUPPORT_PHONE_NUMBER", "+447360494104")}}
	x, err := xml.Marshal(tml)
	if err != nil {
		return nil, err
	}
	return x, nil
}

func VoiceGetReqParam(req []string, p string) string {
	for i, s := range req {
		if strings.HasPrefix(s, p+"=") {
			dig := strings.TrimPrefix(s, p+"=")

			fmt.Printf("i = %v\n", i)
			fmt.Printf("s = %v\n", s)
			return dig
		}
	}
	return ""
}

// VoiceGetProperty ...
// nolint: staticcheck
func VoiceGetProperty(ctx context.Context, ivr string) (string, error) {
	if len(ivr) == 6 {
		resp, err := PropertyService.LookupIvr(ctx, &property.LookupIvrRequest{
			Ivr: wrap.StringValue(&ivr),
		})
		if err != nil {
			return "", err
		}
		return resp.GetProperty().GetId(), nil
	}
	return "", grpcerr.New(grpcerr.InvalidData, "ivr value not equals 6")
}
